{% extends "layout.html" %}

{% comment %}
Copyright (C) 2015-2019  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block title %} Overview &ndash; Software Heritage API {% endblock %}

{% block navbar-content %}
<h4>Web API</h4>
{% endblock %}

{% block content %}
<div class="swh-api-1-doc">
  {% include 'includes/apidoc-header.html' %}
</div>
<script>
  swh.webapp.initPage('api');
</script>
{% endblock %}
