/**
 * Copyright (C) 2018  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.dataTables_filter {
    margin-top: 15px;
}

.dataTables_filter input {
    width: 70%;
    float: right;
}

.dataTables_filter label {
    font-weight: bold !important;
    white-space: inherit !important;
}

.dataTables_wrapper {
    position: static;
}

.swh-table.dataTable {
    border-collapse: collapse !important;
}

.swh-table.dataTable th {
    border-top: none;
}

.swh-table.dataTable tr.selected {
    background: #fecd1b;
}

.page-item.active .page-link {
    color: rgba(0, 0, 0, 0.75);
    background-color: #e9ecef;
    border-color: #dee2e6;
}

.dataTables_scrollBody .swh-table.dataTable thead {
    border-top: none;
}

.dataTables_scrollBody {
    min-height: 300px;
}
